@@ -29,15 +29,15 @@ class MaximaCompletionObject : public Cantor::CompletionObject
 {
   public:
     MaximaCompletionObject( const QString& cmd, int index, MaximaSession* session );
-    ~MaximaCompletionObject() override;
+    ~MaximaCompletionObject() override = default;
 
   protected:
-    bool mayIdentifierContain(QChar c) const Q_DECL_OVERRIDE;
-    bool mayIdentifierBeginWith(QChar c) const Q_DECL_OVERRIDE;
+    bool mayIdentifierContain(QChar c) const override;
+    bool mayIdentifierBeginWith(QChar c) const override;
 
   protected Q_SLOTS:
-    void fetchCompletions() Q_DECL_OVERRIDE;
-    void fetchIdentifierType() Q_DECL_OVERRIDE;
+    void fetchCompletions() override;
+    void fetchIdentifierType() override;
 };
 
 #endif /* _MAXIMACOMPLETIONOBJECT_H */
