@@ -24,9 +24,7 @@
 #include "qalculatehighlighter.h"
 #include "defaultvariablemodel.h"
 
-#include <QTextEdit>
 #include <QProcess>
-#include <QRegExp>
 #include <QRegularExpression>
 
 #include <libqalculate/Calculator.h>
@@ -148,7 +146,6 @@ void QalculateSession::readOutput()
                     m_currentExpression->parseOutput(m_finalOutput);
                     m_finalOutput.clear();
                 }
-
         }
 }
 
@@ -168,47 +165,41 @@ void QalculateSession::storeVariables(QString& currentCmd, QString output)
         This is probably not the best way to get the variable and value.
         But since qalc does not  provide a way to get the list of variables, we will have to stick to parsing
     **/
-    QString value;
-    QString var;
-    QRegExp regex;
+    QRegularExpression regex;
     // find the value
-    regex.setPattern(QLatin1String("[\\s\\w\\W]+=\\s*([\\w\\W]+)"));
-    if(regex.exactMatch(output)) {
-        int pos = regex.indexIn(output);
-        if (pos > -1) {
-            value = regex.cap(1);
-            value = value.trimmed();
-            value.replace(QLatin1String("\n"), QLatin1String(""));
-            value.remove(QLatin1String(">"));
-        }
+    regex.setPattern(QStringLiteral("^[\\s\\w\\W]+=\\s*([\\w\\W]+)$"));
+    QRegularExpressionMatch match = regex.match(output);
+    QString value;
+    if(match.hasMatch()) {
+        value = match.captured(1).trimmed();
+        value.replace(QLatin1String("\n"), QLatin1String(""));
+        value.remove(QLatin1String(">"));
     }
 
     //find the varaiable.
     // ex1: currentCmd = save(10, var_1,category, title): var_1 = variable
     // ex2: currentCmd = save(planet(jupter,mass), jupiter_mass, category, title): jupiter_mass = variable
-    // Not the best regex. Cab be improved
-    regex.setPattern(QLatin1String("\\s*save\\s*\\(\\s*[\\s\\w]+\\s*,([\\s\\w]+),*[\\w\\W]*\\)\\s*;*$|\\s*save\\s*\\(\\s*[\\s\\w\\W]+\\)\\s*,([\\s\\w]+),*[\\w\\W]*\\)\\s*;*$"));
-    if(regex.exactMatch(currentCmd)) {
-        int pos = regex.indexIn(currentCmd);
-        if (pos > -1) {
-            if(!regex.cap(1).trimmed().isEmpty())
-                var =  regex.cap(1).trimmed();
-            else
-                var = regex.cap(2).trimmed();
 
-            var = var.trimmed();
-            var.replace(QLatin1String("\n"), QLatin1String(""));
-            var.remove(QLatin1String(">"));
-        }
+    //  regex.setPattern(QLatin1String("\\s*save\\s*\\(\\s*[\\s\\w]+\\s*,([\\s\\w]+),*[\\w\\W]*\\)\\s*;*$|\\s*save\\s*\\(\\s*[\\s\\w\\W]+\\)\\s*,([\\s\\w]+),*[\\w\\W]*\\)\\s*;*$"));
+
+    regex.setPattern(QStringLiteral("^\\s*save\\s*\\("
+                                    "(?:.+?(?:\\(.+?,.+?\\))|(?:[^,()]+?)),"
+                                    "(.+?),"
+                                    "(?:.+?),"
+                                    "(?:.+?)\\)\\s*;?$"));
+    QString var;
+    match = regex.match(currentCmd);
+    if (match.hasMatch()) {
+        var = match.captured(1).trimmed();
+        var.replace(QLatin1String("\n"), QLatin1String(""));
+        var.remove(QLatin1String(">"));
     }
     if(!value.isEmpty() && !var.isEmpty())
         variables.insert(var, value);
-
 }
 
 void QalculateSession::readError()
 {
-
     QString error =  QLatin1String(m_process->readAllStandardError());
     if(m_currentExpression) {
         m_currentExpression->parseError(error);
@@ -255,13 +246,11 @@ void QalculateSession::interrupt()
 
 void QalculateSession::runExpression()
 {
-
     const QString& command = m_currentExpression->command();
     foreach(const QString& cmd, command.split(QLatin1Char('\n'))) {
         m_commandQueue.enqueue(cmd);
     }
     runCommandQueue();
-
 }
 
 
@@ -274,14 +263,12 @@ void QalculateSession::runCommandQueue()
             m_currentCommand.toLower().trimmed().startsWith(QLatin1String("store")) ||
             m_currentCommand.trimmed().startsWith(QLatin1String("saveVariables"))) {
 
-                    m_currentCommand = parseSaveCommand(m_currentCommand);
+                m_currentCommand = parseSaveCommand(m_currentCommand);
             }
 
-
         m_currentCommand = m_currentCommand.trimmed();
         m_currentCommand += QLatin1String("\n");
         m_process->write(m_currentCommand.toLocal8Bit());
-
     }
 }
 
@@ -295,53 +282,50 @@ QString QalculateSession::parseSaveCommand(QString& currentCmd)
         saveVariables filename
     */
 
-    QRegExp regex;
-    regex.setCaseSensitivity(Qt::CaseInsensitive);
+    QRegularExpression regex;
+    regex.setPatternOptions(QRegularExpression::CaseInsensitiveOption);
 
-    regex.setPattern(QLatin1String("\\s*save\\s*definitions\\s*"));
-    if(regex.exactMatch(currentCmd)) {
+    regex.setPattern(QStringLiteral("^\\s*save\\s*definitions\\s*$"));
+    if(regex.match(currentCmd).hasMatch()) {
         // save the variables in  ~/.cantor/backends/qalculate/definitions
         currentCmd.clear();
         return currentCmd;
     }
 
-    regex.setPattern(QLatin1String("\\s*save\\s*mode\\s*"));
-    if(regex.exactMatch(currentCmd)) {
+    regex.setPattern(QStringLiteral("^\\s*save\\s*mode\\s*$"));
+    if(regex.match(currentCmd).hasMatch()) {
         // save the mode in ~/.cantor/backends/qalculate/cantor_qalc.cfg
         currentCmd.clear();
         return currentCmd;
     }
 
-    regex.setPattern(QLatin1String("\\s*saveVariables\\s*[\\w\\W]+"));
-    if(regex.exactMatch(currentCmd)) {
+    regex.setPattern(QStringLiteral("^\\s*saveVariables\\s*[\\w\\W]+$"));
+    if(regex.match(currentCmd).hasMatch()) {
         // save the variables in a file
         currentCmd.clear();
         return currentCmd;
     }
 
-
-    regex.setPattern(QLatin1String("\\s*store\\s*([a-zA-Z_]+[\\w]*)|\\s*save\\s*([a-zA-Z_]+[\\w]*)"));
-    if(regex.exactMatch(currentCmd)) {
+    regex.setPattern(QStringLiteral("^\\s*store\\s*([a-zA-Z_]+[\\w]*)|\\s*save\\s*([a-zA-Z_]+[\\w]*)$"));
+    QRegularExpressionMatch match = regex.match(currentCmd);
+    if(match.hasMatch()) {
         m_isSaveCommand = true;
-        int pos = regex.indexIn(currentCmd);
-        if(pos > -1) {
-            if(!regex.cap(1).trimmed().isEmpty())
-                currentCmd = QStringLiteral("save(%1, %2)").arg(QStringLiteral("ans")).arg(regex.cap(1).trimmed());
-            else
-                currentCmd = QStringLiteral("save(%1, %2)").arg(QStringLiteral("ans")).arg(regex.cap(2).trimmed());
 
-            return currentCmd;
-        }
+        QString str = match.captured(1).trimmed();
+        if (str.isEmpty())
+            str = match.captured(2).trimmed();
+
+        currentCmd = QStringLiteral("save(%1, %2)").arg(QStringLiteral("ans"), str);
+
+        return currentCmd;
     }
 
-    regex.setPattern(QLatin1String("\\s*save\\s*(\\([\\w\\W]+\\))\\s*;*$"));
-    if(regex.exactMatch(currentCmd)) {
+    regex.setPattern(QStringLiteral("^\\s*save\\s*(\\([\\w\\W]+\\))\\s*;*$"));
+    match = regex.match(currentCmd);
+    if(match.hasMatch()) {
         m_isSaveCommand = true;
-        int pos = regex.indexIn(currentCmd);
-        if (pos > -1) {
-            currentCmd = QStringLiteral("save%1").arg(regex.cap(1).trimmed());
-            return currentCmd;
-        }
+        currentCmd = QStringLiteral("save%1").arg(match.captured(1).trimmed());
+        return currentCmd;
     }
 
     /*
@@ -354,19 +338,19 @@ QString QalculateSession::parseSaveCommand(QString& currentCmd)
     */
     m_saveError =  currentCmd + QLatin1String("\nError: Could not save.\n");
     return QLatin1String("");
-
 }
 
 void QalculateSession::currentExpressionStatusChanged(Cantor::Expression::Status status)
 {
     // depending on the status of the expression change the status of the session;
     switch (status) {
-
         case Cantor::Expression::Computing:
             break;
         case Cantor::Expression::Interrupted:
             changeStatus(Cantor::Session::Done);
             break;
+        case Cantor::Expression::Queued:
+            break;
         case Cantor::Expression::Done:
         case Cantor::Expression::Error:
             qDebug() << " ******  STATUS   " << status;
@@ -380,7 +364,6 @@ void QalculateSession::currentExpressionStatusChanged(Cantor::Expression::Status
 
 Cantor::Expression* QalculateSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave, bool internal)
 {
-
     qDebug() << " ** evaluating expression: " << cmd << endl;
     qDebug() << " size of expression queue: " << m_expressionQueue.size() << endl;
 
@@ -394,7 +377,6 @@ Cantor::Expression* QalculateSession::evaluateExpression(const QString& cmd, Can
     runExpressionQueue();
 
     return expr;
-
 }
 
 void QalculateSession::runExpressionQueue()
@@ -417,7 +399,6 @@ void QalculateSession::runExpressionQueue()
         connect(m_currentExpression, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(currentExpressionStatusChanged(Cantor::Expression::Status)));
         // start processing the expression
         m_currentExpression->evaluate();
-
     }
 }
 
@@ -437,7 +418,6 @@ QSyntaxHighlighter* QalculateSession::syntaxHighlighter(QObject* parent)
     return new QalculateHighlighter(parent);
 }
 
-
 Cantor::DefaultVariableModel* QalculateSession::variableModel() const
 {
     return m_variableModel;
